	/*
 * Copyright 2006 Structured Data, LLC.
 * $Id: TSChartLib.js 41 2006-04-25 13:13:30Z duncan $
 *
 * Version: 0.1
 * 
 * TSChartLib is a chart library (hence the clever name).  For more
 * information, please see our website at http://www.thumbstacks.com.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

function getElementPosition(offsetTrail){
	var offsetLeft = 0;
	var offsetTop = 0;
	while (offsetTrail){
	offsetLeft += offsetTrail.offsetLeft;
	offsetTop += offsetTrail.offsetTop;
	offsetTrail = offsetTrail.offsetParent;
	}
	if (navigator.userAgent.indexOf('Mac') != -1 && typeof document.body.leftMargin != 'undefined'){
	offsetLeft += document.body.leftMargin;
	offsetTop += document.body.topMargin;
	}
	return {left:offsetLeft,top:offsetTop};
} 
 
/******************************************************************************
 * base chart type.  this is kind of a chumpy inheritance model,
 * but it's easier than maintaining everything in different places.
 *****************************************************************************/
function TSChartLib_AbstractChart()
{
	// style constants - IE doesn't support define?
	
	this.STYLE_PLAIN = 		0;
	this.STYLE_3D = 		1;
	this.STYLE_PERCENT = 		2;
	this.STYLE_FILLED = 		4;
	this.STYLE_VALUE_LABELS = 	8;
	
	// fields
	this.lineStyle = "#222";
	this.lineWidth = "2";
	this.fillStyle = "#00f";
	this.shadowStyle = "#00c";
	this.chartStyle = this.STYLE_PLAIN;
	this.axisLabels = null;
	this.axisTitleY = "";
	this.axisTitleX = "";
	this.canvasTop = 0;
	this.canvasLeft = 0;
	this.margins = Array( 8, 8, 8, 8 ); // tlbr

	this.defaultColorScheme = 
	[
		{ color: "#0cf", 	shadowColor: "#09f" }, // blue
		{ color: "#ff0", 	shadowColor: "#fc0" }, // yellow
		{ color: "#c6c", 	shadowColor: "#c3c" }, // purple
		{ color: "#f00", 	shadowColor: "#c00" }, // red
		{ color: "#6c0", 	shadowColor: "#690" }, // green
	];
	
	/**
	 * internal chart structure
	 */
	this.data = Array();

	/**
	 * references to labels, which are dom nodes
	 * (not drawn)
	 */
	this.labels = Array();

	/**
	 *
	 **/
	this.setAxisTiles = function(titx,tity)
	{
		this.axisTitleY = titx;
		this.axisTitleX = tity;
	}
	
	/**
	 * find the x, y position of the canvas
	 * in the document.  this is based on the method
	 * from quirksmode, see
	 * http://www.quirksmode.org/
	 */
	this.find_canvas_position = function( o )
	{
		this.canvasTop = 0;
		this.canvasLeft = 0;
		pos = getElementPosition(o);
		this.canvasTop = pos.top;
		this.canvasLeft = pos.left;		
	};

	/**
	 * reset chart data.
	 */
	this.reset = function()
	{
		
		var i, l = this.data.length;
		for( i = 0; i< l; i++ )
		{
			delete this.data[i];
			this.data[i] = null;
		}
		this.data.splice( 0, l );
		this.resetLabels();
	};

	/**
	 * reset labels: this is a separate method, because it should
	 * be called anytime the chart is redrawn (unlike resetting data)
	 */
	this.resetLabels = function()
	{
		l = this.labels.length;
		for( i = 0; i< l; i++ )
		{
			if( this.labels[i].parentNode )
				this.labels[i].parentNode.removeChild( this.labels[i] );
			delete this.labels[i];
			this.labels[i] = null;
		}
		
		this.labels.splice( 0, l );
	};

	/**
	 * add axis labels (not for pie charts)
	 */
	this.add_labels = function( labels )
	{
		this.axisLabels = labels;
	};
	
	/**
	 * Add a data element - basically, push onto the stack.  values are
	 * data sets - generally arrays of points, but in the case of pie
	 * charts, a single point.  In the case of the pie chart, the value
	 * can be passed as a number; for other charts, it should be an array
	 * of points.
	 *
	 * @param value			the value, as an array of points or as a number
	 * @param label			[OPTIONAL] the label for the element
	 * @param color			[OPTIONAL] the fill color
	 * @param shadowColor	[OPTIONAL] the shadow color, for 3D charts
	 * @param lineColor 	[OPTIONAL] For line charts only, the line color
	 * @param lineWidth		[OPTIONAL] For line charts only, line width
	 */
	this.add_element = function( value, label, color, shadowColor, lineColor, lineWidth )
	{
		if( !value )
		{
//			throw "Invalid entry value";
			return;
		}
		var elt = new Object();
				
		if( typeof( value )=="Array" || ( value.length && value.length>0))
		{
			elt.data = value;
			elt.value = 0;
		}
		else
		{
			elt.value = value;
		}
		if( label ) elt.label = label;
		if( color ) elt.color = color;
		if( shadowColor ) elt.shadowColor = shadowColor;
		if( lineColor ) elt.lineColor = lineColor;
		if( lineWidth ) elt.lineWidth = lineWidth; // can be zero
		
		this.data.push( elt );
	};
		
	/**
	 * draw the chart.
	 *
	 * @param canvas		the canvas element.  we're relying on size,
	 * 						ownerDocument, and parentNode (for labels) -
	 * 						so this needs to be in the DOM.
	 */
	this.render = function( canvas ){};
	
	/**
	 * draw an animated chart.  the parameter is type-dependent,
	 * but will be an integer.  if the chart doesn't support animation
	 * (or the passed variable), it should just call render.
	 * 
	 * @param canvas		the canvas element.  we're relying on size,
	 * 						ownerDocument, and parentNode (for labels) -
	 * 						so this needs to be in the DOM.
	 * @param type			the animation type, if multiple types are supported.
	 */
	this.animate = function( canvas, type )
	{
		this.render( canvas );
	};
}



	/**
	 * Bar chart type, extends AbstractChart
	 */
	function TSChartLib_BarChart()
	{
		// defaults for type 

		this.lineStyle = "#333";
		this.lineWidth = "1";

		this.drawScaleLines = function(canvas,range, w, h, cat, barsize, depth, sep)
		{
			// base
			var ctx = canvas.getContext("2d");
			ctx.lineWidth = 1;
			var div=9;
//			ctx.strokeStyle = this.axisColor;
			var canvasleft	= parseInt(this.canvasLeft);
			var canvasTop	= parseInt(this.canvasTop);
			ctx.fillStyle = "#f8f8f8";
			for (k=0; k<2; k++) {
				ctx.beginPath();
				ctx.moveTo(this.margins[1], h-1 );
				ctx.lineTo(this.margins[1]+depth/2, h -depth);
				ctx.lineTo(w, h -depth);				
				ctx.lineTo(w-depth/2, h-1 );
				ctx.closePath();
				if (k) ctx.stroke();
				else ctx.fill();
			}
			ctx.beginPath();
			ctx.moveTo(this.margins[1]+depth/2, h -depth);
			ctx.lineTo(this.margins[1]+depth/2, 0 );
			ctx.lineTo(w-1, 0 );
			ctx.lineTo(w-1, h-depth );
			ctx.closePath();
			ctx.stroke();
			
			ctx.beginPath();
			ctx.moveTo(this.margins[1], h);
			ctx.lineTo(this.margins[1], depth );
			ctx.lineTo(this.margins[1]+depth/2, 0 );
			ctx.stroke();
			var yPos = 0;
			var step = (parseFloat(h-depth)/div);
			var stepval = (parseFloat(range)/div);
			for  (i=0; i<=div; i++) {
				yPos=h-step*(div-i);
				ctx.beginPath();
				ctx.moveTo(this.margins[1]-5, yPos);
				ctx.lineTo(this.margins[1], yPos);
				ctx.stroke();
				var t = new String();
				var o = canvas.ownerDocument.createElement( "div" );
				this.labels.push( o );
				o.className = "TSChartLib_BarChartLabel";
				// required attributes
				o.style.position = "absolute";
				o.style.textAlign = "right";
				// format?
				t = Math.round(stepval*(div-i)*10)/10;
				o.innerHTML = t;
				canvas.parentNode.appendChild( o );			
				// We have to set the element's position after we put it into the document in order to take effect in IExplorer
				o.style.left = canvasleft-15+ "px";
				o.style.top =  canvasTop+ yPos -7 + "px";
			}
			var step = (parseFloat(w)/(cat+1));
			
			for  (i=0; i<cat; i++) {
				var xpos = (i+1)*(barsize+sep) +this.margins[1];
				ctx.beginPath();
				ctx.moveTo(xpos, h-depth);
				ctx.lineTo(xpos-depth/2, h);
				ctx.stroke();
				var t = new String();
				var o = canvas.ownerDocument.createElement( "div" );
				this.labels.push( o );
				o.className = "TSChartLib_BarChartLabel";
				// required attributes
				o.style.position = "absolute";
				o.style.textAlign = "right";
				// format?
				t = this.axisLabels[i];
				o.innerHTML = t;
				canvas.parentNode.appendChild( o );			
				// We have to set the element's position after we put it into the document in order to take effect in IExplorer
				o.style.left = canvasleft+xpos-barsize/2+ "px";
				o.style.top =  canvasTop+h+"px";
				
			}
		}
		
		this.sort = function(index)
		{
			var values = this.data[index].data;
			var current, i;
			var length = values.length;
			var arraySort = new Array();
			for (var i=0; i<length; i++) {
				arraySort[i] = i;
			}
			for (current=0; current<length-1; current++) {
				for (i=current+1; i<length; i++) { 
					if (values[arraySort[i]] < values[arraySort[current]])   // Intercambiamos indices
					{
						temp = arraySort[i];
						arraySort[i] = arraySort[current];
						arraySort[current] = temp;
					}
				}
			}		
			return  arraySort;
			
		}
		
		
		/**
		 * draw the chart.
		 */
		this.render = function( canvas )
		{
			if( this.data.length <= 0 )
			{
		//		throw "No data for chart";
				return;
			}
			var eps = 0.0001;
			var w, h;
			var min, max, a;
			var numcat = 0;
			var scale = 1;
			var depth = 0.70;  
			var sep = 2.70;  
			var ctx = canvas.getContext("2d");
			var canvasleft	= parseInt(this.canvasLeft);
			w = canvas.offsetWidth;
			h = canvas.offsetHeight;
			this.resetLabels();
					
			this.find_canvas_position( canvas );
			// prep canvas
			ctx.save();
			ctx.clearRect( 0, 0, w, h );
			max = 0;
			min = 0;
			
			// figure out range, scale
			for( i = 0; i< this.data.length; i++ )
			{				
				if( null == numcat ||  this.data.length > numcat ) numcat=  this.data.length;
				a = this.data[i].data;
			
				if( a )
				{
					for( j = 0; j< a.length; j++ )
					{
						if( a[j] != null )
						{
							if( null == max || a[j] > max ) max = a[j];
							if( null == min || a[j] < min ) min = a[j];
						}
					}
				}
			}

			if( min == max || max <= (0 + eps) ) 
			{
			//	throw "Invalid data.";
				return;
			}
			//alert(numcat+ '  width ' +w);
			
			range = max;
			barsize = (w-this.margins[1]- numcat*sep)/(numcat+1);		
			//alert(barsize);
			depth = barsize * depth;			
			scale = (h-depth)/range;
			
			ctx.lineWidth = this.lineWidth;
			ctx.strokeStyle = this.lineStyle;
			ctx.fillStyle = this.fillStyle;
		
			this.drawScaleLines(canvas, range, w, h, numcat, barsize, depth, sep);
			for( i = 0; i< this.data.length; i++ )
			{
				a = this.data[i].data;
				if( a )
				{
					var aOrd = this.sort(i);					
					for( j = a.length-1; j>=0; j-- )
					{						
						if (a[aOrd[j]]*scale > 0)
							this.drawBar((i)*(barsize+sep)+this.margins[1]+barsize/2, barsize, a[aOrd[j]]*scale,h,  depth, aOrd[j], j == a.length-1, ctx);					
					}
				}
			}
			
			ctx.restore();	
		};

		this.drawBar = function(x, width, top, h, depth, color, ultimo, ctx)
		{
			// default stroke
			ctx.strokeStyle = this.lineStyle;
			if( this.data[i].color ) ctx.fillStyle = this.data[color].color;
			else ctx.fillStyle = this.defaultColorScheme[color%this.defaultColorScheme.length].color; 
			ctx.fillRect(x, h-top, width,top);
			ctx.strokeRect(x, h-top, width, top);
			if( this.data[i].color ) ctx.fillStyle = this.data[color].shadowColor;
			else ctx.fillStyle = this.defaultColorScheme[color%this.defaultColorScheme.length].shadowColor; 
			for (k=0; k<ultimo+1; k++) 
			{
			
				ctx.beginPath();
				ctx.moveTo(x+barsize, h );
				ctx.lineTo(x+barsize, h-top );
				ctx.lineTo(x+barsize+depth/2, h-top-depth );
				ctx.lineTo(x+barsize+depth/2, h-depth );				
				ctx.closePath();
				if (k) ctx.stroke();
				else ctx.fill();
				
				if (ultimo)
				{
					ctx.beginPath();
					ctx.moveTo(x, h-top );
					ctx.lineTo(x+depth/2, h-top-depth );
					ctx.lineTo(x+barsize+depth/2, h-top-depth );				
					ctx.lineTo(x+barsize, h-top );
					ctx.closePath();
					if (k) ctx.stroke();
					else ctx.fill();
				}
			}
		};
		
		// inheritance
		this.base = new TSChartLib_AbstractChart();
		for( var i in this.base ) if( !this[i] ) this[i] = this.base[i];
	}


	/******************************************************************************
 * Pie chart type, extends AbstractChart
 *****************************************************************************/
function TSChartLib_PieChart( displayUnit )
{
	// type-specific fields
	this.displayUnit = (this.displayUnit)?this.displayUnit:""; //jva: el nombre de las unidades de la info
	this.rotateBy = 0;
	this.skewFactor = .25;
	
	// animation flags
	this.hideLabels = false;
	this.maxSlices = -1;
	
	// defaults for type (fat lines, white)
	this.lineStyle = "#fff";
	this.lineWidth = "1";

	/**
	 * reset chart data
	 */
	this.reset = function()
	{
		// local 
		this.rotateBy = 0;
		// call base class
		this.base.reset();
	};

	/**
	 * drawLabelsTable
	 **/
	this.drawLabelsTable = function(divLabels)
	{
		//divLabels = document.getElementById("TSChartLib_Labels");
		if( !divLabels ) return;
		var tbl = document.createElement( "Table" );
		tbl.className = "report_table";// configurable style
		tbl.style.height = "15px";

		var rw = tbl.insertRow(0);
		var cll = rw.insertCell(0); cll.className="report_header";
		var cll = rw.insertCell(1); cll.className="report_header";
		cll.innerHTML = this.axisTitleX;
		var cll = rw.insertCell(2); cll.className="report_header";
		cll.innerHTML = this.axisTitleY;

		for( i = 0; i<this.data.length && (this.maxSlices==-1 || i<this.maxSlices ); i++ )
		{
//			if( this.data[i].label || this.chartStyle & this.STYLE_PERCENT )
//			{
				var clr = this.defaultColorScheme[i%this.defaultColorScheme.length].color;
				var clrsqr = 	"<div style='border: #000 1px solid;background-color:"+
					clr+";width:15px;height:10px;'></div>";
				rw = tbl.insertRow(i+1);
				cll = rw.insertCell(0); cll.className="report_default";
				cll.innerHTML= "&nbsp;";
				cll.style.border= "#000 1px solid";
				cll.style.background= clr;
				cll.style.width="15px";
				cll = rw.insertCell(1); cll.className="report_default";
				cll.innerHTML= this.data[i].label;
				cll = rw.insertCell(2); cll.className="report_default";
				unit = (this.displayUnit)?this.displayUnit:"";
				cll.innerHTML = "<b>"+this.data[i].value+unit+"</b>";
//			}
		}
		//this.labels.push( tbl );
		divLabels.appendChild( tbl );
	}

	/**
 	 *  drawOverDiv
	 **/
	this.drawOverDiv = function(	canvas,
					label,lblval,clr,shclr,
					startAngle,endAngle,
					y_offset,y_skew )
	{
		// locate on the screen so we can place labels
		this.find_canvas_position( canvas );
		// set working variables
		var s = Math.min( canvas.offsetWidth, canvas.offsetHeight );
		a = ( startAngle + (( endAngle - startAngle ) /2 )) ;
		/* x and y offsets are relative to a centered origin...*/
		if( this.chartStyle & this.STYLE_3D ) {
			x = (s/2.5)*Math.cos( a * Math.PI / 180 );
			y = y_skew*(s/2.5)*Math.sin( a * Math.PI / 180 )-y_offset;
		}
		else {
			x = s/2 * Math.cos( a * Math.PI / 180 );
			y = s/2 * Math.sin( a * Math.PI / 180 );
		}

		//if( this.axisTitleX ) lblval= this.axisTitleX+":  "+label;
		//if( this.axisTitleY ) label=  this.axisTitleY+":  "+lblval+"  "+this.displayUnit;
		// required attributes
		var xpos = this.canvasLeft + ( canvas.offsetWidth/2 + x );
		var ypos = this.canvasTop  + ( canvas.offsetHeight/2 + y );
		var ovrdiv = canvas.ownerDocument.createElement( "div" );
		this.labels.push( ovrdiv );
		ovrdiv.className = "TSChartLib_PieChartLabel";
		ovrdiv.style.height = "15px";
		ovrdiv.style.position = "absolute";
		ovrdiv.style.border="#000 1px solid";
		ovrdiv.style.background=clr;
		ovrdiv.style.width="5px";
		ovrdiv.style.height="5px";
		ovrdiv.onmouseover= function(){ 
					ol_fgclass="";
					ol_bgclass=""; 
					ol_textfontclass="";
					unit = (this.displayUnit)?this.displayUnit:"";
					txt = label+": "+lblval+" "+unit;
					return overlib(txt,FGCOLOR,clr,BGCOLOR,shclr/*,TEXTCOLOR,shclr*/);
				};
		ovrdiv.onmouseout= function(){ nd(); };
		ovrdiv.appendChild( document.createTextNode("  ") );
		canvas.parentNode.appendChild( ovrdiv );
		// We have to set the element's position after we put it into the document in order to take effect in IExplorer
		ovrdiv.style.left = xpos + "px";
		ovrdiv.style.top = ypos + "px";
	}// drawOverDiv

	/**
	 *	drawLabelText
	 **/
	this.drawLabelText = function(	canvas,label,lblval,clr,startAngle,endAngle,y_offset,y_skew )
	{
		// locate on the screen so we can place labels
		this.find_canvas_position( canvas );
		// set working variables
		var s = Math.min( canvas.offsetWidth, canvas.offsetHeight );
		a = ( startAngle + (( endAngle - startAngle ) /2 )) ;
		/* x and y offsets are relative to a centered origin...*/
		if( this.chartStyle & this.STYLE_3D ) {
			x = (s/2)*Math.cos( a * Math.PI / 180 );
			y = y_skew*(s/2)*Math.sin( a * Math.PI / 180 )-y_offset;
		}
		else {
			x = s/2 * Math.cos( a * Math.PI / 180 );
			y = s/2 * Math.sin( a * Math.PI / 180 );
		}

		// required attributes
		var xpos = this.canvasLeft + ( canvas.offsetWidth/2 + x );
		var ypos = this.canvasTop  + ( canvas.offsetHeight/2 + y );
		var ovrdiv = canvas.ownerDocument.createElement( "div" );
		ovrdiv.className = "TSChartLib_PieChartLabel";
		ovrdiv.style.height = "15px";
		ovrdiv.style.position = "absolute";
		unit = (this.displayUnit)?this.displayUnit:"";
		ovrdiv.innerHTML = "<b>"+label+": "+lblval+" "+unit+"</b>";
		canvas.parentNode.appendChild( ovrdiv );
		// We have to set the element's position after we put it into the document in order to take effect in IExplorer
		ovrdiv.style.left = xpos + "px";
		ovrdiv.style.top = ypos + "px";
	}// drawLabelText
	
	/**
	 * draw 3d effect for the chart
	 */
	this.draw3dSlice = function(ctx,cx,cy,y_offset,radius,startAngle,endAngle)
	{
		if( startAngle<-90 || startAngle>=180 || endAngle<=0 )	return;

		// this is a little funky because of how arcs work in IE
		// (they don't seem to move the path point).  we have to
		// fudge just a bit...
		for( var k=0; k<2; k++ )
		{
			if (/MSIE/.test(navigator.userAgent) && !window.opera)
			{
				ctx.beginPath();
				var a = Math.max( startAngle, 0 ) * Math.PI / 180;
				ctx.moveTo( radius * Math.cos( a ), radius * Math.sin( a ) + y_offset);
				ctx.arc( cx, cy + y_offset, radius,
						Math.min( endAngle, 180 ) * Math.PI / 180,
						Math.max( startAngle, 0 ) * Math.PI / 180,
						true );
				a = Math.max( startAngle, 0 ) * Math.PI / 180;
				ctx.lineTo( radius * Math.cos( a ), radius * Math.sin( a ) );
				a = Math.min( endAngle, 180 ) * Math.PI / 180;
				ctx.lineTo( radius * Math.cos( a ), radius * Math.sin( a ) );
				ctx.closePath();
			}
			else
			{
				ctx.beginPath();
				ctx.arc( cx, cy + y_offset, radius,
						Math.min( endAngle, 180 ) * Math.PI / 180,
						Math.max( startAngle, 0 ) * Math.PI / 180,
						true );
				ctx.arc( cx, cy + 0, radius,
						Math.max( startAngle, 0 ) * Math.PI / 180,
						Math.min( endAngle, 180 ) * Math.PI / 180,
						false );
				ctx.closePath();
			}
			if( k ) ctx.stroke(); else ctx.fill();
		}		
	}// draw3dSlice
	
	/**
	 * draw an slice of the pay
	 */
	this.drawSlice = function(ctx,cx,cy,radius,startAngle,endAngle)
	{		
		//alert( "startAngle="+startAngle+"\nendAngle="+endAngle);
		for( k = 0; k< 2; k++ )
		{
			ctx.beginPath();
			// IE prefers arc method...				
			ctx.arc(cx, cy, radius, 
				startAngle * Math.PI / 180, 
				endAngle * Math.PI / 180, 
				false);
			ctx.lineTo( cx, cy);				
			ctx.closePath();
			if( k ) ctx.stroke(); else ctx.fill();
		}
	}// drawSlice

	/**
	 * draw the chart.
	 */
	this.render = function( canvas )
	{
		if( this.data.length <= 0 )
		{
//			throw "No data for chart";
			return;
		}
				
		var i, startAngle=this.rotateBy-90, endAngle = 0;
		var a, i, j, k, x, y, w, h, s, radius;
		var cx = 0, cy = 0;
		var ctx = canvas.getContext("2d");
		
		var y_skew = 1;
		var y_offset = 0;
		
		this.resetLabels();
		
		w = canvas.offsetWidth;
		h = canvas.offsetHeight;
		
		// make sure it'll fit in the box
		s = Math.min( w, h );
		radius = s * .45;

		// prep canvas
		ctx.save();
		ctx.clearRect( 0, 0, w, h );
		ctx.translate( w * .50, h * .50 );

		if( this.chartStyle & this.STYLE_3D )
		{
			y_skew = 1 - this.skewFactor;
			y_offset = ( 1 - y_skew ) * radius;
			ctx.scale( 1, y_skew );
			ctx.translate( 0, -y_offset );
		}
				
		// style
		ctx.lineWidth = this.lineWidth;
		ctx.strokeStyle = this.lineStyle;
		ctx.fillStyle = this.fillStyle;
		
		// figure out the bits
		var total=0;
		for( i = 0; i< this.data.length; i++ )
			if( this.data[i].value > 0 ) total += this.data[i].value;
		
		// and draw
		for( i = 0; i<this.data.length && (this.maxSlices==-1 || i<this.maxSlices ); i++ )
		{
			if( this.data[i].value <= 0 ) continue;
					
			// find the end angle for this slice.
			endAngle = startAngle + ( 360 * ( this.data[i].value / total ));
			// ensure all values are displayed
			if( startAngle+1.5>endAngle ) endAngle+=1.5;

			// fancy chart style
			if( this.chartStyle & this.STYLE_3D ) 
			{
				// specific parameters?
				ctx.fillStyle = ( this.data[i].shadowColor )?
					this.data[i].shadowColor :
					this.defaultColorScheme[i%this.defaultColorScheme.length].shadowColor;
				// default stroke
				ctx.strokeStyle = this.lineStyle;
				this.draw3dSlice(ctx,cx,cy,y_offset,radius,startAngle,endAngle);
			}

			// specific color parameters?
			ctx.fillStyle = ( this.data[i].color )? 
				this.data[i].color : 
				this.defaultColorScheme[i%this.defaultColorScheme.length].color;
			// default stroke
			ctx.strokeStyle = this.lineStyle;
			// draw the slice
			this.drawSlice(ctx,cx,cy,radius,startAngle,endAngle);
			
			// labels
			if( !this.hideLabels && ( this.data[i].label || this.chartStyle & this.STYLE_PERCENT ))
			{
				var clr = this.defaultColorScheme[i%this.defaultColorScheme.length].color;
				var shclr = this.defaultColorScheme[i%this.defaultColorScheme.length].shadowColor;
				this.drawOverDiv(canvas,
						this.data[i].label,
						this.data[i].value,
						clr,shclr,
						startAngle,endAngle,
						y_offset,y_skew);   
				/* 
				 * Dibuja el texto directamente
				this.drawLabelText(canvas,
						this.data[i].label,
						this.data[i].value,
						clr,
						startAngle,endAngle,
						y_offset,y_skew);
				*/
			}// if labels
			
			startAngle = endAngle;
		}// for
		ctx.restore();
	};
	
	// inheritance
	this.base = new TSChartLib_AbstractChart();
	for( var i in this.base ) if( !this[i] ) this[i] = this.base[i];

}


