/*
 * Copyright 2006 Structured Data, LLC.
 * $Id: TSChartLib.js 41 2006-04-25 13:13:30Z duncan $
 *
 * Version: 0.1
 * 
 * TSChartLib is a chart library (hence the clever name).  For more
 * information, please see our website at http://www.thumbstacks.com.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

function getElementPosition(offsetTrail){
	var offsetLeft = 0;
	var offsetTop = 0;
	while (offsetTrail){
	offsetLeft += offsetTrail.offsetLeft;
	offsetTop += offsetTrail.offsetTop;
	offsetTrail = offsetTrail.offsetParent;
	}
	if (navigator.userAgent.indexOf('Mac') != -1 && typeof document.body.leftMargin != 'undefined'){
	offsetLeft += document.body.leftMargin;
	offsetTop += document.body.topMargin;
	}
	return {left:offsetLeft,top:offsetTop};
} 
 
/******************************************************************************
 * base chart type.  this is kind of a chumpy inheritance model,
 * but it's easier than maintaining everything in different places.
 *****************************************************************************/
function TSChartLib_AbstractChart()
{
	// style constants - IE doesn't support define?
	
	this.STYLE_PLAIN = 		0;
	this.STYLE_3D = 		1;
	this.STYLE_PERCENT = 		2;
	this.STYLE_FILLED = 		4;
	this.STYLE_VALUE_LABELS = 	8;
	
	// fields
	this.lineStyle = "#222";
	this.lineWidth = "2";
	this.fillStyle = "#00f";
	this.shadowStyle = "#00c";
	this.chartStyle = this.STYLE_PLAIN;
	this.axisLabels = null;
	this.axisTitleY = "";
	this.axisTitleX = "";
	this.canvasTop = 0;
	this.canvasLeft = 0;
	this.margins = Array( 8, 8, 8, 8 ); // tlbr

	this.defaultColorScheme = 
	[
		{ color: "#0cf", 	shadowColor: "#09f" }, // blue
		{ color: "#ff0", 	shadowColor: "#fc0" }, // yellow
		{ color: "#c6c", 	shadowColor: "#c3c" }, // purple
		{ color: "#f00", 	shadowColor: "#c00" }, // red
		{ color: "#6c0", 	shadowColor: "#690" }, // green
	];
	
	/**
	 * internal chart structure
	 */
	this.data = Array();

	/**
	 * references to labels, which are dom nodes
	 * (not drawn)
	 */
	this.labels = Array();

	/**
	 *
	 **/
	this.setAxisTiles = function(titx,tity)
	{
		this.axisTitleY = titx;
		this.axisTitleX = tity;
	}
	
	/**
	 * find the x, y position of the canvas
	 * in the document.  this is based on the method
	 * from quirksmode, see
	 * http://www.quirksmode.org/
	 */
	this.find_canvas_position = function( o )
	{
		this.canvasTop = 0;
		this.canvasLeft = 0;
		pos = getElementPosition(o);
		this.canvasTop = pos.top;
		this.canvasLeft = pos.left;		
	};

	/**
	 * reset chart data.
	 */
	this.reset = function()
	{
		var i, l = this.data.length;
		for( i = 0; i< l; i++ )
		{
			delete this.data[i];
			this.data[i] = null;
		}
		this.data.splice( 0, l );
		this.resetLabels();
	};

	/**
	 * reset labels: this is a separate method, because it should
	 * be called anytime the chart is redrawn (unlike resetting data)
	 */
	this.resetLabels = function()
	{
		l = this.labels.length;
		for( i = 0; i< l; i++ )
		{
			if( this.labels[i].parentNode )
				this.labels[i].parentNode.removeChild( this.labels[i] );
			delete this.labels[i];
			this.labels[i] = null;
		}
		this.labels.splice( 0, l );
	};

	/**
	 * add axis labels (not for pie charts)
	 */
	this.add_labels = function( labels )
	{
		this.axisLabels = labels;
	};
	
	/**
	 * Add a data element - basically, push onto the stack.  values are
	 * data sets - generally arrays of points, but in the case of pie
	 * charts, a single point.  In the case of the pie chart, the value
	 * can be passed as a number; for other charts, it should be an array
	 * of points.
	 *
	 * @param value			the value, as an array of points or as a number
	 * @param label			[OPTIONAL] the label for the element
	 * @param color			[OPTIONAL] the fill color
	 * @param shadowColor	[OPTIONAL] the shadow color, for 3D charts
	 * @param lineColor 	[OPTIONAL] For line charts only, the line color
	 * @param lineWidth		[OPTIONAL] For line charts only, line width
	 */
	this.add_element = function( value, label, color, shadowColor, lineColor, lineWidth )
	{
		if( !value )
		{
			throw "Invalid entry value";
			return;
		}
		var elt = new Object();
				
		if( typeof( value )=="Array" || ( value.length && value.length>0))
		{
			elt.data = value;
			elt.value = 0;
		}
		else
		{
			elt.value = value;
		}
		if( label ) elt.label = label;
		if( color ) elt.color = color;
		if( shadowColor ) elt.shadowColor = shadowColor;
		if( lineColor ) elt.lineColor = lineColor;
		if( lineWidth ) elt.lineWidth = lineWidth; // can be zero
		
		this.data.push( elt );
	};
		
	/**
	 * draw the chart.
	 *
	 * @param canvas		the canvas element.  we're relying on size,
	 * 						ownerDocument, and parentNode (for labels) -
	 * 						so this needs to be in the DOM.
	 */
	this.render = function( canvas ){};
	
	/**
	 * draw an animated chart.  the parameter is type-dependent,
	 * but will be an integer.  if the chart doesn't support animation
	 * (or the passed variable), it should just call render.
	 * 
	 * @param canvas		the canvas element.  we're relying on size,
	 * 						ownerDocument, and parentNode (for labels) -
	 * 						so this needs to be in the DOM.
	 * @param type			the animation type, if multiple types are supported.
	 */
	this.animate = function( canvas, type )
	{
		this.render( canvas );
	};
}
