/******************************************************************************
 * Bar chart type, extends AbstractChart
 *****************************************************************************/
function TSChartLib_BarChart(displayUnit)
{
	// defaults for type 
	this.displayUnit = (displayUnit)?displayUnit:""; //jva: el nombre de las unidades de la info
	this.lineStyle = "#FFF";
	this.lineWidth = "1";
	this.axisColor = "#999";
	// animation fields
	this.barScale = 1.0;
	this.depthScale = 1.0;
	
	/**
	* getScaleUnit
	*/
	this.getScaleUnit = function(range)
	{
		var scaleUnit = 1;
		if( range >= 10 ) {
			while( range / scaleUnit > 8 ) scaleUnit *= 10;
			if( range / scaleUnit < 4 ) scaleUnit *= .5;
			if( range / scaleUnit < 4 ) scaleUnit *= .5;
		}
		else {// very small?
			while( range / scaleUnit < 8 ) scaleUnit /= 10;
			if( range / scaleUnit > 4 ) scaleUnit /= .5;
			if( range / scaleUnit > 4 ) scaleUnit /= .5;
		}
		return scaleUnit;
	}; // getScaleUnit
	
	/**
	* drawScaleLines
	*/
	this.drawScaleLines = function(canvas,max,min,scale,scaleUnit)
	{
		var ctx = canvas.getContext("2d");
		var w = parseInt( canvas.width );
		var h = parseInt( canvas.height );
		var x = Math.floor( min / scaleUnit ) * scaleUnit;
		
		var canvasleft	= parseInt(this.canvasLeft);
		var marginleft	= parseInt(this.margins[1]);
		var canvastop	= parseInt(this.canvasTop);
		var margintop	= parseInt(this.margins[2]);
		
		ctx.lineWidth = 1;
		ctx.strokeStyle = this.axisColor;
		ctx.beginPath();
	
		while( x<=max )
		{
			if( x>min ) 
			{
				ctx.moveTo( this.margins[1] + ( x - min ) * scale, this.margins[0] );
				ctx.lineTo( this.margins[1] + ( x - min ) * scale, h - this.margins[2] );
				
				// labels?				
				if( !this.hideLabels && ( this.chartStyle & this.STYLE_VALUE_LABELS ))
				{										
					var xpos	= ( canvasleft + marginleft + ( x - min ) * scale );
					var ypos	= ( canvastop + h - margintop );
					var t = new String();
					var o = canvas.ownerDocument.createElement( "div" );
					o.className = "TSChartLib_ChartLabel";
					// required attributes
					o.style.position = "absolute";
					o.style.textAlign = "right";
					// format?
					t = x;
					o.innerHTML = t.toFixed(2);
					canvas.parentNode.appendChild( o );			
					// We have to set the element's position after we put it into the document in order to take effect in IExplorer
					o.style.left = xpos + "px";
					o.style.top =  ypos + "px";
				}
			}
			x += scaleUnit;
		}
	
		ctx.stroke();
	}; // drawScaleLines
	
	/**
	* draw all the bars
	*/
	this.drawBars = function(canvas,zero, scale, barsize, spacer, depth)
	{
		var ctx = canvas.getContext("2d");
		var w = canvas.width;
		var h = canvas.height;
		
		// style		
		ctx.lineWidth = this.lineWidth;
		ctx.strokeStyle = this.lineStyle;
		ctx.fillStyle = this.fillStyle;
		// draw the bars.  draw negatives first; then the
		// axis, then  positives (this lets us draw the 3d
		// axis on top of the negative bars)
		for( i = 0; i< this.data.length; i++ )
		{
			dtaVal = this.data[i].value;
			if( dtaVal<0 ) this.drawBar( i, dtaVal, ctx, zero, scale, barsize, spacer, depth );
		}
			
		// axis
		ctx.fillStyle = this.axisColor;
		if( this.chartStyle & this.STYLE_3D )
		{
			var df = ( -depth * this.depthScale );
			for( k = 0; k< 2; k++ )
			{
				ctx.beginPath();
				ctx.moveTo( zero, this.margins[0] );
				ctx.lineTo( zero, h - this.margins[2] );
				ctx.lineTo( df + zero, df + h - this.margins[2] );
				ctx.lineTo( df + zero, df + this.margins[0] );
				ctx.closePath();
				if( k ) ctx.stroke();
				else ctx.fill();
			}
		}
		else
		{
			ctx.beginPath();
			ctx.moveTo( zero, this.margins[0] );
			ctx.lineTo( zero, h - this.margins[2] );
			ctx.stroke();
		}
	
		// positives
		for( i=0; i<this.data.length; i++ )
		{
			dtaVal = this.data[i].value;
			if( dtaVal>=0 ) this.drawBar( i, dtaVal, ctx, zero, scale, barsize, spacer, depth );
		}
	}; // drawBars
	
	/**
	* drawLabelsTable
	**/
	this.drawLabelsTable = function( divLabels )
	{
		//divLabels = document.createElement("Div");
		if( !divLabels ) return;
		var tbl = document.createElement( "Table" );
		tbl.className = "report_table";// configurable style
		tbl.style.height = "15px";

		var rw = tbl.insertRow(0);
		var cll = rw.insertCell(0); cll.className="report_header";
		var cll = rw.insertCell(1); cll.className="report_header";
		cll.innerHTML = this.axisTitleX;
		var cll = rw.insertCell(2); cll.className="report_header";
		cll.innerHTML = this.axisTitleY;

		for( i = 0; i<this.data.length; i++ )
		{
			var clr = this.defaultColorScheme[i%this.defaultColorScheme.length].color;
			var clrsqr = 	"<div style='border: #000 1px solid;background-color:"+
					clr+";width:15px;height:10px;'></div>";
			rw = tbl.insertRow(i+1);
			cll = rw.insertCell(0); cll.className="report_default";
			cll.innerHTML= "&nbsp;";
			cll.style.border= "#000 1px solid";
			cll.style.background= clr;
			cll.style.width="15px";
			cll = rw.insertCell(1); cll.className="report_default";
			cll.innerHTML= this.data[i].label;
			cll = rw.insertCell(2); cll.className="report_default";
			cll.innerHTML = "<b>"+this.data[i].value+this.displayUnit+"</b>";
		}
		//this.labels.push( tbl );
		divLabels.appendChild( tbl );
	} //drawLabelsTable
	
	/**
	* createOverDiv
	*/
	this.createOverDiv = function (canvas,left,top,clr,label,val)
	{
		var ovrdiv = canvas.ownerDocument.createElement( "div" );
		//ovrdiv.className = "TSChartLib_ChartLabel";
		ovrdiv.style.height = "15px";
		ovrdiv.style.position = "absolute";
		ovrdiv.style.border="#000 1px solid";
		ovrdiv.style.background=clr;
		ovrdiv.style.width="7px";
		ovrdiv.style.height="7px";
		//if( this.axisTitleX ) lblval= this.axisTitleX+":  "+label;
		//if( this.axisTitleY ) label=  this.axisTitleY+":  "+lblval+"  "+this.displayUnit;
		var caption = 	"<div style=\"border:1px solid #000; width:15px; height:15px; background-color:"+clr+
				"; display:inline; float:left;\"></div> "+
				" &nbsp;<b>"+label+"</b>: "+val;
		ovrdiv.onmouseover= function(){ return overlib(caption); };
		//ovrdiv.onmouseover= function(){ return overlib(val,CAPTION,label); };
		ovrdiv.onmouseout= function(){ nd(); };
		ovrdiv.appendChild( document.createTextNode("    ") );
		canvas.parentNode.appendChild( ovrdiv );
		// We have to set the element's position after we put it into the document in order to take effect in IExplorer
		ovrdiv.style.left = left + "px"
		ovrdiv.style.top = top + "px";
	};
	
	/**
	* draw the chart.
	*/
	this.drawBarLabels = function(canvas,min,scale,spacer,barsize)
	{
		// locate on the screen so we can place labels
		this.find_canvas_position( canvas );
		for( i=0; i<this.data.length; i++ ) {
			if( this.data[i].label ) {
				var clr = this.defaultColorScheme[i%this.defaultColorScheme.length].color;
				var label = this.data[i].label;
				val = this.data[i].value;
				
				var left = ( this.canvasLeft + this.margins[1] + ( val-min )*scale );
				var top= ( this.canvasTop + this.margins[0] + spacer + barsize*i );
				this.createOverDiv(canvas,left,top,clr,label,val);
			}
		}
	}; // drawBarLabels

	/**
	 * draw the chart.
	 */
	this.render = function( canvas )
	{
		if( this.data.length <= 0 ) {
			throw "No data for chart";
			return;
		}

		this.resetLabels();
		// locate on the screen so we can place labels
		this.find_canvas_position( canvas );
		
		var cx = 0, cy = 0;
		var barSize = 1;
		var range = 0, scale = 0, scaleUnit = 1, zero = 0;
		var depth = 0;
		var w = canvas.width;
		var h = canvas.height;
		
		// prep canvas
		var ctx = canvas.getContext("2d");
		ctx.save();
		ctx.clearRect( 0, 0, w, h );
				
		// figure out range, scale
		var max = 0, min = 0;
		for( i=0; i<this.data.length; i++ ) {
			var dtaVal = this.data[i].value;
			if( dtaVal ) {
				if( dtaVal>max ) max = dtaVal;
				if( dtaVal<min ) min = dtaVal;
			}
		}
		// pad a bit
		range = ( max - min );
		max += (range * .10);
		min -= ( range * .10 );
		range = ( max - min );

		// bad data?
		if( range <= 0 ) {
			throw "Range values are zero or less: "+range;
			return;
		}
		
		scale = (( w - ( this.margins[1] + this.margins[3] )) / range );
		zero = this.margins[1] - ( scale * min );
		scaleUnit = this.getScaleUnit(range);
		
		spacer = h - ( this.margins[0] + this.margins[2] );
		barsize = spacer/(this.data.length+2);
		spacer= barsize;
		depth= barsize/2.5;
		// draw scale lines
		this.drawScaleLines(canvas,max,min,scale,scaleUnit);
		// draw the bars
		this.drawBars(canvas,zero, scale, barsize, spacer, depth);
		// axis labels?
		if( !this.hideLabels )
			this.drawBarLabels(canvas,min,scale,spacer,barsize);
		ctx.restore();
		
	}; // render

	/**
	 * drawBar
	 */
	this.drawBar = function( i, val, ctx, zero, scale, barsize, spacer, depth )
	{
		var v, x, y;
		var df =( this.chartStyle & this.STYLE_3D )? -depth*this.depthScale : 0;
		
		// default stroke
		ctx.strokeStyle = this.lineStyle;
			
		v = val * this.barScale;
		y = df + this.margins[0] + spacer + ( barsize * i );

		if( v < 0 )
		{
			x = df + zero + (v*scale);
			if( this.chartStyle & this.STYLE_3D )
			{
				if( this.data[i].shadowColor ) 
					ctx.fillStyle = this.data[i].shadowColor ;
				else {
					var clrIdx = i%this.defaultColorScheme.length;
					ctx.fillStyle = this.defaultColorScheme[clrIdx].shadowColor;
				}
				
				// bar
				for( k = 0; k< 2; k++ )
				{
					ctx.beginPath();
					ctx.moveTo( x, y + barsize );
					ctx.lineTo( x - df, y - df + barsize );
					ctx.lineTo( zero, y - df + barsize );
					ctx.lineTo( zero, y + barsize );
					if( k ) ctx.stroke();
					else ctx.fill();
				}
			}
				
			// specific parameters?
			if( this.data[i].color ) 
				ctx.fillStyle = this.data[i].color;
			else {
				var clrIdx = i%this.defaultColorScheme.length;
				ctx.fillStyle = this.defaultColorScheme[clrIdx].color;
			}

			ctx.fillRect( x, y, -v * scale , barsize );
			ctx.strokeRect( x, y, -v * scale , barsize );
			
		}
		else
		{
			x = df + zero;
			
			if( this.chartStyle & this.STYLE_3D )
			{
				// shadow
						
				if( this.data[i].shadowColor ) ctx.fillStyle = this.data[i].shadowColor ;
				else ctx.fillStyle = this.defaultColorScheme[i%this.defaultColorScheme.length].shadowColor;

				// fill separately, stroke once
			
				// bar 
				ctx.beginPath();
				ctx.moveTo( x, y + barsize );
				ctx.lineTo( x - df, y + barsize - df );
				ctx.lineTo( x - df + v * scale, y + barsize - df );
				ctx.lineTo( x + v * scale, y + barsize );
				ctx.fill();
				
				// cap
				ctx.beginPath();
				ctx.moveTo( x + v * scale, y + barsize );
				ctx.lineTo( x - df + v * scale, y + barsize - df );
				ctx.lineTo( x - df + v * scale, y - df );
				ctx.lineTo( x + v * scale, y );
				ctx.fill();
				
				// line
				ctx.beginPath();
				ctx.moveTo( x, y + barsize );
				ctx.lineTo( x - df, y + barsize - df );
				ctx.lineTo( x - df + v * scale, y + barsize - df );
				ctx.lineTo( x + v * scale, y + barsize );
				ctx.moveTo( x + v * scale, y );
				ctx.lineTo( x - df + v * scale, y - df );
				ctx.lineTo( x - df + v * scale, y + barsize - df );
				ctx.stroke();
				
			}
				
			// specific parameters?
			if( this.data[i].color ) ctx.fillStyle = this.data[i].color;
			else ctx.fillStyle = this.defaultColorScheme[i%this.defaultColorScheme.length].color;
		
			ctx.fillRect( x, y , v * scale , barsize );
			ctx.strokeRect(  x, y, v * scale , barsize );
			
		}

	};
	
	 	
	this.animate = function( canvas, type )
	{
		var p = this;
		if( !type ) type = 0;
		
		switch( type )
		{
		case 1:
			this.depthScale = 0;
			break;
		case 2:
			this.depthScale = 0;
			this.barScale = 0;
			break;
		default:
			this.barScale = 0;
			break;
		}
		this.render( canvas );
		setTimeout( function(){ p.next_step( canvas, type ); }, 20 );
	};
		
	this.next_step = function( canvas, type )
	{
		var p = this;
		if( !type ) type = 0;
		switch( type )
		{
		case 1:
			this.depthScale += 0.25;
			break;
		case 2:
			this.barScale += .25;
			this.depthScale += 0.25;
			break;
		default:
			this.barScale += .25;
			break;
		}
		if( this.barScale > 1.0 ) this.barScale = 1.0;
		if( this.depthScale > 1.0 ) this.depthScale = 1.0;
		this.render( canvas );
		if( this.barScale < 1.0 || this.depthScale < 1.0 )
		{
			setTimeout( function(){ p.next_step( canvas, type ); }, 20 );
		}
	};
	 
	
	// inheritance
	this.base = new TSChartLib_AbstractChart();
	for( var i in this.base ) if( !this[i] ) this[i] = this.base[i];
}
