/******************************************************************************
 * Line chart type, extends AbstractChart
 *****************************************************************************/
function TSChartLib_LineChart()
{
	// defaults for type 
	this.lineStyle = "#333";
	this.lineWidth = "2";
	this.margins = Array( 8, 18, 18, 8 ); // tlbr

	// marker styles
	this.MARKER_SQUARE = 	1;
	this.MARKER_TRIANGLE = 	2;
	this.MARKER_DIAMOND =	4;
	this.MARKER_CIRCLE =	8;
	
	// fields
	this.chartMin = null;
	this.markerStyle = 0;
	
	// animation flags
	this.maxPoints = -1;
	this.maxSets = -1;
	this.dataCount = 0;
	this.displayUnit = "";

	/**
	* createOverDiv
	*/
	this.createOverDiv = function (canvas,left,top,clr,label,val)
	{
		var ovrdiv = canvas.ownerDocument.createElement( "div" );
		//ovrdiv.className = "TSChartLib_ChartLabel";
		ovrdiv.style.height = "15px";
		ovrdiv.style.position = "absolute";
		ovrdiv.style.width="7px";
		ovrdiv.style.height="7px";
		var caption = 	"<b>"+label+"</b>: "+val;
		ovrdiv.onmouseover= function(){ return overlib(caption); };
		//ovrdiv.onmouseover= function(){ return overlib(val,CAPTION,label); };
		ovrdiv.onmouseout= function(){ nd(); };
		ovrdiv.appendChild( document.createTextNode("    ") );
		canvas.parentNode.appendChild( ovrdiv );
		// We have to set the element's position after we put it into the document in order to take effect in IExplorer
		ovrdiv.style.left = left + "px";
		ovrdiv.style.top = top + "px";
	};
	
	/**
	* drawLabelsTable
	**/
	this.drawLabelsTable = function(divLabels)
	{
		if( !divLabels ) return;
		var tbl = document.createElement( "Table" );
		tbl.className = "report_table";// configurable style
		tbl.style.height = "15px";

		var rw = tbl.insertRow(0);
		var cll = rw.insertCell(0); cll.className="report_header";
		cll.innerHTML = this.axisTitleX;
		var cll = rw.insertCell(1); cll.className="report_header";
		cll.innerHTML = this.axisTitleY;
		for( j = 0; j<this.data.length; j++ )
		{
			for( i=0; i<this.data[j].data.length; i++ )
			{
				rw = tbl.insertRow(i+1);
				cll = rw.insertCell(0); cll.className="report_default";
				var jmax = this.data.length;
				cll.innerHTML= this.axisLabels[ j*jmax+i ];
				cll = rw.insertCell(1); cll.className="report_default";
				cll.innerHTML = "<b>"+this.data[j].data[i]+this.displayUnit+"</b>";
				divLabels.appendChild( tbl );
			}
		}
	}; // drawLabelsTable

	/**
	* draw the mark points
	*/
	this.drawMarkPoints = function(canvas,a,space,scale,min)
	{
		if( !this.markerStyle ) return;
		
		var w = canvas.width;
		var h = canvas.height;
		var ctx = canvas.getContext("2d");
		ctx.fillStyle = ctx.strokeStyle;
		ctx.strokeStyle = "#eee";
		ctx.lineWidth = 1;
				
		for( i = 0; i< a.length; i++ )
		{
			if( this.maxPoints > -1 && this.maxPoints <= i ) break;
			if( a[i]==null ) continue;
			x = this.margins[1] + space * ( i + .5 );
			y = h - this.margins[2] - ( scale * ( a[i] - min ));
					
			switch( this.markerStyle )
			{
				case this.MARKER_CIRCLE:
					ctx.beginPath();
					ctx.arc( x, y, 4, 0, Math.PI * 2, false );
					ctx.fill();
					ctx.beginPath();
					ctx.arc( x, y, 4, 0, Math.PI * 2, false );
					ctx.stroke();
				break;
				case this.MARKER_DIAMOND:
					ctx.beginPath();
					ctx.moveTo( x - 4, y );
					ctx.lineTo( x, y - 4 );
					ctx.lineTo( x + 4, y );
					ctx.lineTo( x, y + 4 );
					ctx.fill();
					ctx.beginPath();
					ctx.moveTo( x - 4, y );
					ctx.lineTo( x, y - 4 );
					ctx.lineTo( x + 4, y );
					ctx.lineTo( x, y + 4 );
					ctx.stroke();
				break;
				case this.MARKER_TRIANGLE:
					ctx.beginPath();
					ctx.moveTo( x, y - 5 );
					ctx.lineTo( x - 4, y + 3 );
					ctx.lineTo( x + 4, y + 3 );
					ctx.closePath();
					ctx.fill();
					ctx.beginPath();
					ctx.moveTo( x, y - 5 );
					ctx.lineTo( x - 4, y + 3 );
					ctx.lineTo( x + 4, y + 3 );
					ctx.closePath();
					ctx.stroke();
				break;
				default:
					ctx.fillRect( x - 3, y - 3, 6, 6 );
					ctx.strokeRect( x - 3, y - 3, 6, 6 );
				break;
			}
		}
	}; // draw Mark Points
	
	/**
	* draw the chart lines.
	*/
	this.drawLines = function (canvas,space,scale,min)
	{
		var ctx = canvas.getContext("2d");
		var w = canvas.width;
		var h = canvas.height;
		var kct = ( this.chartStyle & this.STYLE_FILLED ) ? 2 : 1 ;
		for( j = 0; j<this.data.length ; j++ )
		{
			if( !this.data[j].data || !this.data[j].data.length ) continue;
			a = this.data[j].data;
			
			if( this.data[j].color ) ctx.fillStyle = this.data[j].color;
			else ctx.fillStyle = this.defaultColorScheme[j%this.defaultColorScheme.length].color;
			
			if( this.data[j].lineColor ) ctx.strokeStyle = this.data[j].lineColor;
			else ctx.strokeStyle = this.lineStyle;
		
			if( this.data[j].lineWidth != null ) ctx.lineWidth = this.data[j].lineWidth;
			else ctx.lineWidth = this.lineWidth;
		
			
			for( k = 0; k< kct; k++ )
			{
				ctx.beginPath();
				
				// find first point (skip nulls)				
				for( i = 0; i< a.length; i++ )
				{
					if( this.maxPoints > -1 && this.maxPoints <= i ) break;
					
					if( null!=a[i] )
					{
						x = this.margins[1] + space * ( i + .5 );
						if( kct == 1 || k )
						{
							y = h-this.margins[2] - ( scale*( a[i]-min ));
							ctx.moveTo( this.margins[1], y );
							ctx.lineTo( x, y);
						}
						else
						{
							y = h-this.margins[2] - ( scale*( a[i]-min ));
							ctx.moveTo( this.margins[1], h-this.margins[2] );
							ctx.lineTo( this.margins[1], y );
							ctx.lineTo( x, y);
						}
						break;
					}
				}
					
				// now draw the rest of the points (skipping nulls)
				for( ++i; i< a.length; i++ )
				{
					if( this.maxPoints > -1 && this.maxPoints <= i ) break;

					if( null != a[i] )
					{
						x = this.margins[1] + space * ( i + .5 );
						y = h - this.margins[2] - ( scale * ( a[i] - min ));
						ctx.lineTo( x, y);
					}
				}
					
				if( kct == 1 || k )
				{
					ctx.lineTo( w-this.margins[1],  y);
				}
				else
				{					
					ctx.lineTo( w-this.margins[1],  y);
					ctx.lineTo( w-this.margins[1], h - this.margins[2] );
				}
				
				
				
				if( kct == 1 || k ) ctx.stroke();
				else ctx.fill();
			}
			if( !this.markerStyle ) this.markerStyle= this.MARKER_CIRCLE;
			this.drawMarkPoints(canvas,a,space,scale,min);
		}
	}; // drawLines

	/**
	* draw Vertical axis labels
	*/
	this.drawYAxisLabels = function(canvas,min,max,scale,scaleUnit)
	{
		y = Math.floor( min / scaleUnit ) * scaleUnit;
		// locate on the screen so we can place labels
		this.find_canvas_position( canvas );
		var ctx = canvas.getContext("2d");
		var w = parseInt( canvas.width );
		var h = parseInt( canvas.height );
		
		while( y <= max )
		{			
			if( y<min && y!=0 ) { y += scaleUnit; continue; }
			
			ctx.lineWidth = 1;
			ctx.strokeStyle = "rgba( 128, 128, 128, 0.25 )";
			if( y == 0 )	ctx.strokeStyle = "rgb( 0, 0, 0 )";
			
			ctx.beginPath();
			ctx.moveTo( this.margins[1], h - this.margins[2] - ( ( y - min ) * scale ));
			ctx.lineTo( w - this.margins[3], h - this.margins[2] - ( ( y - min ) * scale ));
			ctx.stroke();
			
			for( i=0; i<2; i++)
			if( !this.hideLabels && (this.chartStyle & this.STYLE_VALUE_LABELS) )
			{
				var t = new String();
				var o = canvas.ownerDocument.createElement( "div" );
				o.className = "TSChartLib_ChartLabel";
				o.style.background = "none";
				if( i==0 ) o.style.color= "#FFF";
					
				// required attributes
				o.style.position = "absolute";
				o.style.textAlign = "right";
				o.style.width = "100px";
				t = y.toFixed(2);
				o.innerHTML = t;
				this.labels.push( o );
				canvas.parentNode.appendChild( o );
				// We have to set the element's position after we put it into the document 
				// in order to take effect in IExplorer
				o.style.left = ( this.canvasLeft - 50 + this.margins[1] - 2 +i) + "px";
				o.style.top = ( parseInt(this.canvasTop) + h - 20 -
						parseInt(this.margins[2]) - ( (y-min)*scale ) +i) + "px";
			}
			y += scaleUnit;
		}
	};// drawYAxisLabels
	
	/**
	* draw horizontal axis labels
	*/
	this.drawXAxisLabels = function(canvas,min,max,scale,scaleUnit)
	{
		y = Math.floor( min / scaleUnit ) * scaleUnit;
		// locate on the screen so we can place labels
		this.find_canvas_position( canvas );
		var ctx = canvas.getContext("2d");
		var w = canvas.width;
		var h = canvas.height;
		
		ctx.lineWidth = 2;
		ctx.strokeStyle = "rgba( 128, 128, 128, 0.75 )";
		ctx.beginPath();
				
		ct = this.dataCount;
		// calculate horizontal spacing
		space = ( ( w - this.margins[1] - this.margins[3] ) / ct );
		
		// Draw verical axis
		x = this.margins[1];
		y = h - this.margins[2];
		ctx.moveTo( x, y );
		ctx.lineTo( x, 0 );
		
		// Draw all labels
		for( i = 0; i< ct; i++ )
		{
			x = this.margins[1] + ( space * ( i + 0.5 ));
			y = h - this.margins[2];
			ctx.moveTo( x, y );
			ctx.lineTo( x, y + 7 );
			
			if( this.axisLabels && this.axisLabels[i] && !this.hideLabels )
			{
				yy = h - this.margins[2]-( (a[i]- min)*scale);
				this.createOverDiv(canvas,
						this.canvasLeft + ( x ) - 5,
						this.canvasTop + ( yy ) - 5,
						"rgba( 255, 128, 128,255)",
						this.axisLabels[i],a[i]
				);
					
				this.createOverDiv(
						canvas,
						this.canvasLeft + ( x-5 ),
						this.canvasTop  + ( y+2 ),
						"rgba( 255, 128, 128,255)",
						"", this.axisLabels[i]
				);
			}
		}
		ctx.stroke();
		ctx.restore();
	}; // drawXAxisLabels
	
	/**
	 * draw the chart.
	 */
	this.render = function( canvas )
	{
		if( this.data.length <= 0 )
		{
			throw "No data for chart";
			return;
		}
		if( !this.displayUnit ) this.displayUnit="";
		
		var a, i, j, k, x, y, w, h, s;
		var ctx = canvas.getContext("2d");
		var max = 0, min = 0, range = 0, scale = 0, zero = 0, ct = 0;
		var space = 0;
		var last = 0;
		var scaleUnit = 1;
				
		this.resetLabels();
		
		w = canvas.width;
		h = canvas.height;
		
		// prep canvas
		ctx.save();
		ctx.clearRect( 0, 0, w, h );
				
		// style
		ctx.lineWidth = this.lineWidth;
		ctx.strokeStyle = this.lineStyle;
		ctx.fillStyle = this.fillStyle;
		
		// figure out data range
		for( j = 0; j< this.data.length; j++ )
		{
			if( !this.data[j].data || !this.data[j].data.length ) continue;
			a = this.data[j].data;
			for( i = 0; i< a.length; i++ )
			{
				if( ( !i && !j ) || a[i] < min ) min = a[i];	
				if( ( !i && !j ) || a[i] > max ) max = a[i];	
			}
			if( a.length > ct ) ct = a.length;
		}
		
		this.dataCount = ct;
		max *= 1.05;
		
		if( this.chartMin != null ) min = this.chartMin;
		
		if( min < 0 ) min *= 1.05;
		else if( min > 0 ) min *= .95;
		
		range = ( max - min );
		
		if( ct <= 0 || range == 0 )
		{
			throw "Invalid data";
		}
	
		scale = ( ( h - this.margins[0] - this.margins[2] ) / range );

		// draw scale... how can we figure out the appropriate scale range?

		scaleUnit = 1;
		if( range >= 10 )
		{
			while( range / scaleUnit > 8 ) scaleUnit *= 10;
			if( range / scaleUnit < 4 ) scaleUnit *= .5;
			if( range / scaleUnit < 4 ) scaleUnit *= .5;
		}
		else // very small?
		{
			while( range / scaleUnit < 8 ) scaleUnit /= 10;
			if( range / scaleUnit > 4 ) scaleUnit /= .5;
			if( range / scaleUnit > 4 ) scaleUnit /= .5;
		}

		// calculate horizontal spacing		
		space = ( ( w - this.margins[1] - this.margins[3] ) / ct );
		// draw lines...
		this.drawLines(canvas,space,scale,min);
		// draw labels
		this.drawYAxisLabels(canvas,min,max,scale,scaleUnit);
		this.drawXAxisLabels(canvas,min,max,scale,scaleUnit);
	};
	
	this.animate = function( canvas, type )
	{
		var p = this;
		this.maxPoints = 0;
		this.hideLabels = true;
		//this.maxSets = 1;
		this.render( canvas );
		setTimeout( function(){ p.next_step( canvas, type ); }, 20 );
	};
	
	this.next_step = function( canvas, type )
	{
		var p = this;
		this.maxPoints++;
		if( this.maxPoints >= this.dataCount )
		{
			this.hideLabels = false;
		}
		this.render( canvas );
		if( this.maxPoints < this.dataCount )
		{
			setTimeout( function(){ p.next_step( canvas, type ); }, 20 );
		}
		else if( this.maxSets >= 0 && this.maxSets < this.data.length )
		{
			this.maxPoints = 0;
			this.maxSets++;
			setTimeout( function(){ p.next_step( canvas, type ); }, 20 );
		}
	};
	
	// inheritance
	this.base = new TSChartLib_AbstractChart();
	for( var i in this.base ) if( !this[i] ) this[i] = this.base[i];
	
}
