/******************************************************************************
 * Pie chart type, extends AbstractChart
 *****************************************************************************/
function TSChartLib_PieChart( displayUnit )
{
	// type-specific fields
	this.displayUnit = (displayUnit)?displayUnit:""; //jva: el nombre de las unidades de la info
	this.rotateBy = 0;
	this.skewFactor = .25;
	
	// animation flags
	this.hideLabels = false;
	this.maxSlices = -1;
	
	// defaults for type (fat lines, white)
	this.lineStyle = "#fff";
	this.lineWidth = "1";

	/**
	 * reset chart data
	 */
	this.reset = function()
	{
		// local 
		this.rotateBy = 0;
		// call base class
		this.base.reset();
	};

	/**
	 * drawLabelsTable
	 **/
	this.drawLabelsTable = function(divLabels)
	{
		//divLabels = document.getElementById("TSChartLib_Labels");
		if( !divLabels ) return;
		var tbl = document.createElement( "Table" );
		tbl.className = "report_table";// configurable style
		tbl.style.height = "15px";

		var rw = tbl.insertRow(0);
		var cll = rw.insertCell(0); cll.className="report_header";
		var cll = rw.insertCell(1); cll.className="report_header";
		cll.innerHTML = this.axisTitleX;
		var cll = rw.insertCell(2); cll.className="report_header";
		cll.innerHTML = this.axisTitleY;

		for( i = 0; i<this.data.length && (this.maxSlices==-1 || i<this.maxSlices ); i++ )
		{
//			if( this.data[i].label || this.chartStyle & this.STYLE_PERCENT )
//			{
				var clr = this.defaultColorScheme[i%this.defaultColorScheme.length].color;
				var clrsqr = 	"<div style='border: #000 1px solid;background-color:"+
					clr+";width:15px;height:10px;'></div>";
				rw = tbl.insertRow(i+1);
				cll = rw.insertCell(0); cll.className="report_default";
				cll.innerHTML= "&nbsp;";
				cll.style.border= "#000 1px solid";
				cll.style.background= clr;
				cll.style.width="15px";
				cll = rw.insertCell(1); cll.className="report_default";
				cll.innerHTML= this.data[i].label;
				cll = rw.insertCell(2); cll.className="report_default";
				cll.innerHTML = "<b>"+this.data[i].value+this.displayUnit+"</b>";
//			}
		}
		//this.labels.push( tbl );
		divLabels.appendChild( tbl );
	}

	/**
 	 *  drawOverDiv
	 **/
	this.drawOverDiv = function(	canvas,
					label,val,clr,
					startAngle,endAngle,
					y_offset,y_skew )
	{
		// locate on the screen so we can place labels
		this.find_canvas_position( canvas );
		// set working variables
		var s = Math.min( canvas.offsetWidth, canvas.offsetHeight );
		a = ( startAngle + (( endAngle - startAngle ) /2 )) ;
		/* x and y offsets are relative to a centered origin...*/
		if( this.chartStyle & this.STYLE_3D ) {
			x = (s/2.5)*Math.cos( a * Math.PI / 180 );
			y = y_skew*(s/2.5)*Math.sin( a * Math.PI / 180 )-y_offset;
		}
		else {
			x = s/2 * Math.cos( a * Math.PI / 180 );
			y = s/2 * Math.sin( a * Math.PI / 180 );
		}

		//if( this.axisTitleX ) val= this.axisTitleX+":  "+label;
		//if( this.axisTitleY ) label=  this.axisTitleY+":  "+val+"  "+this.displayUnit;
		// required attributes
		var xpos = this.canvasLeft + ( canvas.offsetWidth/2 + x );
		var ypos = this.canvasTop  + ( canvas.offsetHeight/2 + y );
		var ovrdiv = canvas.ownerDocument.createElement( "div" );
		ovrdiv.className = "TSChartLib_ChartLabel";
		ovrdiv.style.height = "15px";
		ovrdiv.style.position = "absolute";
		ovrdiv.style.border="#000 1px solid";
		ovrdiv.style.background=clr;
		ovrdiv.style.width="5px";
		ovrdiv.style.height="5px";
		var caption = 	"<div style=\"border:1px solid #000; width:15px; height:15px; background-color:"+clr+
				"; display:inline; float:left;\"></div> "+
				" &nbsp;<b>"+label+"</b>: "+val;
		ovrdiv.onmouseover= function(){ return overlib(caption); };
		//ovrdiv.onmouseover= function(){ return overlib(val,CAPTION,label); };
		ovrdiv.onmouseout= function(){ nd(); };
		ovrdiv.appendChild( document.createTextNode("  ") );
		canvas.parentNode.appendChild( ovrdiv );
		// We have to set the element's position after we put it into the document in order to take effect in IExplorer
		ovrdiv.style.left = xpos + "px";
		ovrdiv.style.top = ypos + "px";
	}// drawOverDiv

	/**
	 * draw 3d effect for the chart
	 */
	this.draw3dSlice = function(ctx,cx,cy,y_offset,radius,startAngle,endAngle)
	{
		if( startAngle<-90 || startAngle>=180 || endAngle<=0 )	return;

		// this is a little funky because of how arcs work in IE
		// (they don't seem to move the path point).  we have to
		// fudge just a bit...
		for( var k=0; k<2; k++ )
		{
			if (/MSIE/.test(navigator.userAgent) && !window.opera)
			{
				ctx.beginPath();
				var a = Math.max( startAngle, 0 ) * Math.PI / 180;
				ctx.moveTo( radius * Math.cos( a ), radius * Math.sin( a ) + y_offset);
				ctx.arc( cx, cy + y_offset, radius,
						Math.min( endAngle, 180 ) * Math.PI / 180,
						Math.max( startAngle, 0 ) * Math.PI / 180,
						true );
				a = Math.max( startAngle, 0 ) * Math.PI / 180;
				ctx.lineTo( radius * Math.cos( a ), radius * Math.sin( a ) );
				a = Math.min( endAngle, 180 ) * Math.PI / 180;
				ctx.lineTo( radius * Math.cos( a ), radius * Math.sin( a ) );
				ctx.closePath();
			}
			else
			{
				ctx.beginPath();
				ctx.arc( cx, cy + y_offset, radius,
						Math.min( endAngle, 180 ) * Math.PI / 180,
						Math.max( startAngle, 0 ) * Math.PI / 180,
						true );
				ctx.arc( cx, cy + 0, radius,
						Math.max( startAngle, 0 ) * Math.PI / 180,
						Math.min( endAngle, 180 ) * Math.PI / 180,
						false );
				ctx.closePath();
			}
			if( k ) ctx.stroke(); else ctx.fill();
		}		
	}// draw3dSlice
	
	/**
	 * draw an slice of the pay
	 */
	this.drawSlice = function(ctx,cx,cy,radius,startAngle,endAngle)
	{		
		//alert( "startAngle="+startAngle+"\nendAngle="+endAngle);
		for( k = 0; k< 2; k++ )
		{
			ctx.beginPath();
			// IE prefers arc method...				
			ctx.arc(cx, cy, radius, 
				startAngle * Math.PI / 180, 
				endAngle * Math.PI / 180, 
				false);
			ctx.lineTo( cx, cy);				
			ctx.closePath();
			if( k ) ctx.stroke(); else ctx.fill();
		}
	}// drawSlice

	/**
	 * draw the chart.
	 */
	this.render = function( canvas )
	{
		if( this.data.length <= 0 )
		{
			throw "No data for chart";
			return;
		}
				
		var i, startAngle=this.rotateBy-90, endAngle = 0;
		var a, i, j, k, x, y, w, h, s, radius;
		var cx = 0, cy = 0;
		var ctx = canvas.getContext("2d");
		
		var y_skew = 1;
		var y_offset = 0;
		
		this.resetLabels();
		
		w = canvas.offsetWidth;
		h = canvas.offsetHeight;
		
		// make sure it'll fit in the box
		s = Math.min( w, h );
		radius = s * .45;

		// prep canvas
		ctx.save();
		ctx.clearRect( 0, 0, w, h );
		ctx.translate( w * .50, h * .50 );

		if( this.chartStyle & this.STYLE_3D )
		{
			y_skew = 1 - this.skewFactor;
			y_offset = ( 1 - y_skew ) * radius;
			ctx.scale( 1, y_skew );
			ctx.translate( 0, -y_offset );
		}
				
		// style
		ctx.lineWidth = this.lineWidth;
		ctx.strokeStyle = this.lineStyle;
		ctx.fillStyle = this.fillStyle;
		
		// figure out the bits
		var total=0;
		for( i = 0; i< this.data.length; i++ )
			if( this.data[i].value > 0 ) total += this.data[i].value;
		
		// and draw
		for( i = 0; i<this.data.length && (this.maxSlices==-1 || i<this.maxSlices ); i++ )
		{
			if( this.data[i].value <= 0 ) continue;
					
			// find the end angle for this slice.
			endAngle = startAngle + ( 360 * ( this.data[i].value / total ));
			// ensure all values are displayed
			if( startAngle+1.5>endAngle ) endAngle+=1.5;

			// fancy chart style
			if( this.chartStyle & this.STYLE_3D ) 
			{
				// specific parameters?
				ctx.fillStyle = ( this.data[i].shadowColor )?
					this.data[i].shadowColor :
					this.defaultColorScheme[i%this.defaultColorScheme.length].shadowColor;
				// default stroke
				ctx.strokeStyle = this.lineStyle;
				this.draw3dSlice(ctx,cx,cy,y_offset,radius,startAngle,endAngle);
			}

			// specific color parameters?
			ctx.fillStyle = ( this.data[i].color )? 
				this.data[i].color : 
				this.defaultColorScheme[i%this.defaultColorScheme.length].color;
			// default stroke
			ctx.strokeStyle = this.lineStyle;
			// draw the slice
			this.drawSlice(ctx,cx,cy,radius,startAngle,endAngle);
			
			// labels
			if( !this.hideLabels && ( this.data[i].label || this.chartStyle & this.STYLE_PERCENT ))
			{
				var clr = this.defaultColorScheme[i%this.defaultColorScheme.length].color;
				this.drawOverDiv(canvas,
						this.data[i].label,
						this.data[i].value,
						clr,
						startAngle,endAngle,
						y_offset,y_skew);
			}// if labels
			
			startAngle = endAngle;
		}// for
		ctx.restore();
	};
	
	/**
	 * animate.  this can be extended with different values for
	 * type, which is an integer flag.
	 */
	this.animate = function( canvas, type )
	{
		var p = this;
		if( !type ) type = 0;
		switch( type )
		{
		case 1: // this is the "rising 3d" type

			if( ! this.chartStyle & this.STYLE_3D )
			{
				this.render( canvas );
				return;
			}
			// set starting point
			this.hideLabels = true;
			this.skewFactor = 0;
			this.render( canvas );
			
			// start loop
			setTimeout( function(){ p.next_step( canvas, type ); }, 10 );
			
			break;
		
		case 2: // this is a combined type (notice the cascade)

			this.skewFactor = 0;
			if( !this.chartStyle & this.STYLE_3D ) type = 0;

		default: // step through slices...
		
			this.hideLabels = true;
			this.maxSlices = 0;
			setTimeout( function(){ p.next_step( canvas, type ); }, 250 );
			break;
		}
	};
	
	/**
	 * callback method for the animation loop
	 */
	this.next_step = function( canvas, type )
	{
		if( !type ) type = 0;
		var p = this;
			
		switch( type )
		{
		case 1:

			this.skewFactor += 0.05;
			if( this.skewFactor >= 0.3 )
			{
				this.skewFactor = 0.3;
				this.hideLabels = false;
			}
			
			this.render( canvas );
			if( this.skewFactor < 0.3 ) setTimeout( function(){ p.next_step( canvas, type ); }, 10 );
			
			break;
		
		default:
			
			this.maxSlices++;
			if( this.maxSlices >= this.data.length )
			{
				if( type == 2 ) type = 1;
				else this.hideLabels = false;
			}
			this.render( canvas );
			if( this.maxSlices < this.data.length )
				setTimeout( function(){ p.next_step( canvas, type ); }, 250 );
			else if( type == 1 ) // switch to the other animation
				setTimeout( function(){ p.next_step( canvas, type ); }, 10 );
			
		}

	};
	
	// inheritance
	this.base = new TSChartLib_AbstractChart();
	for( var i in this.base ) if( !this[i] ) this[i] = this.base[i];

}
